-------------------------------------------------------------------------------
-- An example circuit type DM74185 renamed Bin_BCD_6bit
-- Minimised equations SoP for an VHDL plan A design 
-- https://digsys.upc.edu/csd/P02/Bin_BCD_6bitA/bin_BCD_6bit.html
---------------------------------------------------
-- Exercise P2  - CSD - Design of standard circuits
-- http://digsys.upc.edu
------------------------------------------------------------------------------
-------------------------------------------------------------------------------

LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;

entity Bin_BCD_6bit is

	port (
		E_L		: 	in STD_LOGIC;
		B		: 	in STD_LOGIC_VECTOR(5 DOWNTO 0); 
		T,U		: 	out STD_LOGIC_VECTOR(3 DOWNTO 0)
	);

end Bin_BCD_6bit;
	
architecture Equations_SoP of Bin_BCD_6bit is

begin

-- T3= E_L
T(3) <= E_L;

-- T2= E_L + B5·B4 + B5·B3
T(2) <=   E_L             or 
        ( B(5) and B(4) ) or 
		( B(5) and B(3) );

-- T1= E_L'·B5·B4'·B3' + E_L'·B5'·B4·B3 + E_L'·B5'·B4·B2 + E_L'·B4·B3·B2

T(1) <= ( not(E_L) and B(5) and not(B(4)) and not(B(3))    ) or 
		( not(E_L) and not(B(5)) and B(4) and B(3)         ) or 
		( not(E_L) and not(B(5)) and B(4) and B(2)         ) or 
        ( not(E_L) and B(4) and B(3) and B(2)              ); 


-- T0= E_L + B5·B3'·B1 + B5'·B4'·B3·B2 + B5'·B4'·B3·B1 + 

T(0) <=   E_L                                                or
        ( B(5) and not(B(3)) and B(1)                      ) or 
        ( not(B(5)) and not(B(4)) and B(3) and B(2)        ) or 
        ( not(B(5)) and not(B(4)) and B(3) and B(1)        ) or 
		
-- + B5'·B3·B2·B1 + B5'·B4·B3'·B2' + B5·B4·B3·B2' + B5·B4'·B3' + B5·B3'·B2
        ( not(B(5)) and B(3) and B(2) and B(1)             ) or 
        ( not(B(5)) and B(4) and not(B(3)) and not(B(2))   ) or
        ( B(5) and B(4) and B(3) and not(B(2))             ) or 
        ( B(5) and not(B(4)) and not(B(3))                 ) or 		
        ( B(5) and not(B(3)) and B(2)                      );		
		
-- U3= E_L + B5·B4'·B3'·B2·B1 + B5'·B4'·B3·B2'·B1' + B5'·B4·B3·B2·B1' +
U(3) <=   E_L                                                             or
        ( B(5) and not(B(4)) and not(B(3)) and B(2) and B(1)            ) or 
        ( not(B(5)) and not(B(4)) and B(3) and not(B(2)) and not(B(1))  ) or 
        ( not(B(5)) and B(4) and B(3) and B(2) and not(B(1))            ) or

-- + B5·B4·B3'·B2'·B1' + B5'·B4·B3'·B2'·B1 + B5·B4·B3·B2'·B1		
        ( B(5) and B(4) and not(B(3)) and not(B(2)) and not(B(1))       ) or 		
        ( not(B(5)) and B(4) and not(B(3)) and not(B(2)) and B(1)       ) or		
        ( B(5) and B(4) and B(3) and not(B(2)) and B(1)                 );  

-- U2= E_L + B4'·B3'·B2·B1' + B5'·B4'·B2·B1 + B5·B4'·B3·B2 + B5'·B4·B2'·B1' + 
U(2) <=   E_L                                                         or
        ( not(B(4)) and not(B(3)) and B(2) and not(B(1))            ) or 
        ( not(B(5)) and not(B(4)) and B(2) and B(1)                 ) or 
        ( B(5) and not(B(4)) and B(3) and B(2)                      ) or
        ( not(B(5)) and B(4) and not(B(2)) and not(B(1))            ) or 
		
--  + B4·B3·B2'·B1' + B5'·B4·B3·B2' + B5·B4'·B3'·B2'·B1 + B5·B4·B3'·B2·B1
        ( B(4) and B(3) and not(B(2)) and not(B(1))                 ) or 		
        ( not(B(5)) and B(4) and B(3) and not(B(2))                 ) or		
        ( B(5) and not(B(4)) and not(B(3)) and not(B(2)) and B(1)   ) or 
        ( B(5) and B(4) and not(B(3)) and B(2) and B(1)             ); 


-- U1= E_L'·B5'·B4·B3'·B2'·B1' + E_L'·B5·B4·B3·B2'·B1' + E_L'·B5'·B4·B3·B2'·B1 +
U(1) <= ( not(E_L) and not(B(5)) and B(4) and not(B(3)) and not(B(2)) and not(B(1)) ) or 
		( not(E_L) and B(5) and B(4) and B(3) and not(B(2)) and not(B(1))           ) or 
		( not(E_L) and not(B(5)) and B(4) and B(3) and not(B(2)) and B(1)           ) or 

-- +  E_L'·B5'·B4'·B3·B2·B1' + E_L'·B5·B4'·B3'·B1' + E_L'·B5·B3'·B2·B1' +
        ( not(E_L) and not(B(5)) and not(B(4)) and B(3) and B(2) and not(B(1))      ) or 
        ( not(E_L) and B(5) and not(B(4)) and not(B(3)) and not(B(1))               ) or
        ( not(E_L) and B(5) and not(B(3)) and B(2) and not(B(1))                    ) or

-- + E_L'·B5'·B4'·B3'·B1 + E_L'·B5·B4'·B3·B1 + E_L'·B5'·B3'·B2·B1 + E_L'·B5·B3·B2·B1		
        ( not(E_L) and not(B(5)) and not(B(4)) and not(B(3)) and B(1)               ) or 		
        ( not(E_L) and B(5) and not(B(4)) and B(3) and B(1)                         ) or 
        ( not(E_L) and not(B(5)) and not(B(3)) and B(2) and B(1)                    ) or 
        ( not(E_L) and B(5) and B(3) and B(2) and B(1)                              ); 

-- U0= E_L + B0
U(0) <= E_L or B(0); 

end Equations_SoP;
								
